//Datei: DS1821.C
// Hauptprogramm
// DS1821 Programmer mit PIC16F84
//
// Holger Klabunde
// Copyright 2000
// 22.04.2001
// Compiler PICCLite

// Verwendete LCD-Anzeige 1x16, 2x16, 2x8 Zeichen

#include "prozess.h"      /* Der verwendete Prozessor */

#include "ds1821.h"
#include "protos.h"

 // Watchdog off, Power up Timer on, XT Oscillator, Code Protect off
 __CONFIG(0x3FF1);

//Globale Flags und Rckgabevariablen

unsigned char stelle; 	//Position auf der LCD-Anzeige
unsigned char key;	//Zum Lesen der Tastatur
unsigned char tastbuf;  //Tastaturpuffer fr lesen im INT
unsigned char boing;    //Warten auf INT
unsigned char rep;      //Mehrfachauslsen vermeiden
unsigned char mode;     //TH oder TL programmieren ?
unsigned char level;    //Schaltpolaritt vom DS1821
unsigned char buf;

char temp;
unsigned char tl,th;     //Temperatur low,high

/* Hauptprogramm */
void main()
{
 InitPIC(); //Interrupts und so weiter
 LCDInit(); //LCD Anzeige einstellen
 LCDCls();  //Anzeige lschen

 mode=0; //Erst Einschalttemperatur eingeben
         //Danach Ausschalttemperatur eingeben
         //Zum Schlu Schaltlevel eingeben

 LCDWrite("Insert");
 LCDPos(2,1);
 LCDWrite("DS1821");
 
 boing=0;
 rep=0;   //Merkt sich den letzen Wert von boing

 GIE=1; //Ints zulassen

 while(1) // Anfang Programm Endlosschleife
  {

   key=0; //Letzten Tastenwert lschen !! Sonst Mehrfachauslsung

   if(boing>0) //Taste gedrckt ?
    {
     if(rep!=boing) //War ein neuer INT da ?
      {
       rep=boing;     //nchstes Auslsen erst wieder wenn neuer INT da war

       //Einzelauslsung
       if(boing==2) key=tastbuf; //Tastaturpuffer lesen, Lowaktiv

       //Ab hier schnelles Tastaturrepeat
       if(boing>20 && (boing&0x07)==0) key=tastbuf;  
      }
    }
   else rep=0; //Taste losgelassen, Tastenwiederholung lschen
   
   if(key==RIGHT && mode<3)
    {
     if(stelle<5) stelle=stelle+1; // Eins nach rechts
     if(stelle==5) stelle=2; // Ans linke Ende des Displays bei berlauf
     ShowDisplay(); 		//Display updaten
    } 

   if(key==UP)
    {
     if(mode<3) //Temperatur eingeben
      {
       if(stelle==4 && temp<125) temp++;   //eins hochzhlen
       if(stelle==3 && temp<116) temp+=10; //zehn hochzhlen
       if(stelle==2 && temp<26)  temp+=100; //hundert hochzhlen
       ShowDisplay(); 		//Display updaten
      } 
     
     if(mode==3) //Schaltpolaritt auf High gendert
      {
       level=1;
       LCDPos(1,7);
       LCDWriteByte(0x31); //Eine 1
      }
    }//if key==up

   if(key==DOWN)
    {
     if(mode<3) //Temperatur eingeben
      {
       if(stelle==4 && temp>-55) temp-=1;   //einer runterzhlen
       if(stelle==3 && temp>-46) temp-=10; //zehner runterzhlen
       if(stelle==2 && temp>45) temp-=100; //hunderter runterzhlen
       ShowDisplay(); 		//Display updaten
      }

     if(mode==3) //Schaltpolaritt auf Low gendert
      {
       level=0;
       LCDPos(1,7);
       LCDWriteByte(0x30); //Eine 0
      }
    }//if key==down

//Hier Enter Taste auswerten
   if(key==ENTER)
    {
     if(mode==0) //Enter-Taste nach einsetzen DS1821 gedrckt
      {
       //Alte Werte aus dem DS auslesen
       ResetThermostat(); //Evtl. von Thermostat auf 1-Wire umschalten

       buf=MasterReset();
       SendByte(READHIGH); //Einschalttemperatur lesen
       th=ReadByte();

       buf=MasterReset();
       SendByte(READLOW); //Ausschalttemperatur lesen
       tl=ReadByte();

       buf=MasterReset();
       SendByte(READSTATUS); //Einschaltlevel lesen
       level=ReadByte();

       level>>=1;   //Bit an die richtige Stelle schieben
       level&=0x01; //Nur Bit D0 stehen lassen 

       GIE=1; //Ints erlauben

       temp=th; //Wert fr th auf Anzeige
       LCDCls();
       LCDPos(2,1);
       LCDWrite("T-On");
       stelle=4; //Cursor auf 1 Grad Stelle
       ShowDisplay();
 	  
      }

     if(mode==1) //Einschalttemperatur wurde eingegeben
      {
       th=(unsigned char)temp; //Wert merken
       temp=tl;  //Wert fr tl auf Anzeige
       
       LCDPos(2,1); //Nchsten Menpunkt anzeigen
       LCDWrite("T-Off");
       ShowDisplay();
      } 

     if(mode==2) //Ausschalttemperatur wurde eingegeben
      {
       tl=(unsigned char)temp; //Wert merken
       LCDCls(); //Nchsten Menpunkt anzeigen
       LCDWrite("Level ");
       LCDWriteByte(0x30+level); //Aktuellen Level anzeigen
      }
      
     if(mode==3) //Schaltpolaritt wurde eingegeben, DS programmieren
      {
       LCDCls();
       //Der DS antwortet komischerweise nicht mehr wenn ich
       //hier ankomme !!! Deshalb die Holzhammermethode
       //Ausschalten, warten, einschalten und programmieren
       DQ=0;
       DSPOWER=0;
       Delay1ms(255); Delay1ms(255); Delay1ms(255); Delay1ms(255); //1s warten

       ResetThermostat();

       //DS1821 Einschalttemperatur schreiben
       buf=MasterReset();
       SendByte(WRITEHIGH);
       SendByte(th);
       Delay1ms(50); //Warten auf Programmierung
       
       //DS1821 Ausschalttemperatur schreiben
       buf=MasterReset();
       SendByte(WRITELOW);
       SendByte(tl);
       Delay1ms(50); //Warten auf Programmierung

       //DS1821 Schaltpolaritt schreiben
       buf=MasterReset();
       SendByte(WRITESTATUS);
       if(level==0) SendByte(0x44); //Low aktiv
       else SendByte(0x46);           //High aktiv
       Delay1ms(50); //Warten auf Programmierung

       //Wir haben noch etwas Speicher !!
       //Also wie es sich gehrt Verify aller Werte

       //DS1821 Einschalttemperatur lesen
       buf=MasterReset();
       SendByte(READHIGH);
       buf=ReadByte();
       if(buf!=th) { LCDWrite("Err TH"); goto DS_ERROR; }

       //DS1821 Ausschalttemperatur lesen
       buf=MasterReset();
       SendByte(READLOW);
       buf=ReadByte();
       if(buf!=tl) { LCDWrite("Err TL"); goto DS_ERROR; }

       //DS1821 Schaltlevel lesen
       buf=MasterReset();
       SendByte(READSTATUS);
       buf=ReadByte();
       buf>>=1;        //Bit auf richtige Stelle schieben
       buf&=0X01;      //Nur Bit D0 stehen lassen
       if(buf!=level) { LCDWrite("Err LV"); goto DS_ERROR; }

       //Meldung DS entnehmen + Reset !
       LCDWrite("Ready");

DS_ERROR:
       LCDPos(2,1);
       LCDWrite("Reset !");
       DQ=0;
       DSPOWER=0;
       while(1); //Endlosschleife bis zum Reset
      }

     mode++; //Nchster Menpunkt
    }

  } // Ende Programm Endlosschleife
}

void ResetThermostat()
{
  while(GIE) GIE=0; //Ints verbieten

  DSPOWER=1; //DS einschalten
  DQ=1;      //Pullup Datenleitung

  buf=MasterReset(); //Nachsehen ob ein DS1821 vorhanden ist
  if(buf>0) //Meldet sich nicht, knnte als Thermostat laufen
   {
    //Jetzt versuchen von Thermostat auf 1-Wire umzuschalten
    DSPOWER=0; //DS abschalten mit DQ=1 !!
    for(buf=0; buf<16; buf++) { DQ=0; DQ=1; };
    DSPOWER=1; //DS wieder einschalten
    Delay1ms(10);

    buf=MasterReset(); //Nochmal nachsehen ob ein DS1821 vorhanden ist
    if(buf>0) //Wieder nix
     {
      LCDCls();
      LCDWrite("DS1821??"); //Fehlermeldung
      LCDPos(2,1);
      LCDWrite("Reset !");
      DQ=0; //Spannungen abschalten
      DSPOWER=0;
      while(1); //Warten auf Reset
     }

   } //if buf>0
}

//************************************************
// Zeigt die 3 Dezimalstellen auf dem LCD Display 
//************************************************
void ShowDisplay()
{
 unsigned char nib;
 
 LCDPos(1,1);
 if(temp<0) //Temperatur negativ ?
  {
   //Dann Wert positiv machen
   B0=~temp; //Einerkomplement
   B0++; //Zweierkomplement
   LCDWriteByte(0x2D); //Vorzeichen -
  }
 else //Temperatur positiv
  {
   B0=temp;
   LCDWriteByte(0x2B); //Vorzeichen +
  } 

 BinToBCD8(); //char zu BCD
 LCDWriteByte(R0+0x30); //Hunderter

 nib=R1>>4; //High-Nibble=> Zehner
 nib+=0x30; //ASCII Zahl
 LCDWriteByte(nib); //Und schreiben

 nib=R1&0x0F; //Low-Nibble=> Einer
 nib+=0x30; //ASCII Zahl
 LCDWriteByte(nib);

 LCDWriteByte(0xDF); //"" Einheit anzeigen
 LCDPos(1,stelle); // Cursor auf alte Dezimalstelle
}
